% This file simulates the version of the model under a Blanchard-Yaari structure. 
%The values of parameters coincide with those at
% table 1 in Lustenhouwer and Mavromatis 2021.

% Authors Joep Lustenhouwer and Kostas Mavromatis, 2022.

ii = 1;

for ll = 0.94:0.04:0.98

tt = 3  % if tt = 1: then spending cut announcement, if tt = 2: labor income tax cut announcement, if tt = 3: transfers cut announcement

parameters betta eta lambda sigma mup zeta ybar cbar phipi phiy rhor phib omegap omegaw betta eta rhog rhotau taubar gbar; 
parameters gamma1 gamma2 delta kappa theta theta_w markup_p markup_w kappa_p kappa_w wbar ibar kbar hbar ybar bbar Rbar Rss rkbar miubar DELTA PHI tauLumpSumBar;

var c y h i i_o q k k_o pi mc pi_w r rk w b z tau_w tau tau_r g rr miu;

varexo eps_g eps_tauw eps_taur;

betta = 0.99;
DELTA = ll;
eta = 1.5;
lambda = 0.0;
sigma = 2;
markup_p = 2;
theta = markup_p/(markup_p-1);
omegap = 0.8;
omegaw = 0.000000001;
zeta = 0.2;
gamma1 = 1/(betta) - 1 + 0.025;
gamma2 = 0.05;
delta = 0.025;
kappa = 17;
markup_w = 2;
theta_w = markup_w/(markup_w-1);
kappa_p = (1-omegap*betta*DELTA)*(1-omegap)/omegap;
kappa_w = (1-omegaw*betta*DELTA)*(1-omegaw)/omegaw;
phipi = 1.5;
phiy = 0.2;
phib = 0.06;
rhog = 0.97;
rhotau = 0.97;
taubar = 0.4;    % Labour income tax rate
gbar = 0.2;      % Government spending as a share of GDP                                              
bbar = 0.7;      % Debt to GDP ratio
wbar = ((theta-1)/theta)^(1/(1-zeta))*zeta^(zeta/(1-zeta))*(1-zeta)*gamma1^(-zeta/(1-zeta));
kbar = (((1-gbar)*((gamma1/wbar*(1-zeta)/zeta)^(1-zeta) - delta/(1-gbar)))^(-sigma)*(1-taubar)*wbar*(gamma1/wbar*(1-zeta)/zeta)^(-eta)*(theta_w-1)/theta_w)^(1/(sigma+eta));
ibar = delta*kbar;
ybar = (gamma1/wbar*(1-zeta)/zeta)^(1-zeta)*kbar;
cbar = ybar*(1-gbar)-ibar;
hbar = gamma1/wbar*(1-zeta)/zeta*kbar;
mup = wbar*hbar*(1-taubar)/cbar;
miubar = 1/(1-betta*DELTA);
rkbar = gamma1;

err = 1;
Rss = 1.01;
while err>0.0001;
tauLumpSumBar = ((Rss-1)/Rss)*bbar - taubar*wbar*hbar/ybar + gbar; % steady state Lump-Sum taxes as a share of GDP.
PHI = sigma*(1 + (1/(betta*Rss))*((sigma-1)/sigma));
Rbar = 1/betta + ((1-DELTA)/(betta*DELTA*miubar*cbar))*(bbar + rkbar*kbar);
err = Rbar - Rss;
Rss = Rbar;
end;




model(linear);

%1 Aggregate Euler Equation
c = (1 - ((1-DELTA)/(sigma*betta*Rbar*DELTA*miubar*sigma))*(bbar+rkbar*kbar))*c(+1) - ((1-DELTA)/(sigma*betta*Rbar*DELTA*miubar*sigma))*(bbar+rkbar*kbar)*miu(+1) - (1/sigma)*(r-pi(+1)) + ((1-DELTA)/(sigma*betta*Rbar*DELTA*miubar*sigma))*(ybar*(b-pi(+1)) + rkbar*kbar*K + rkbar*kbar*rk + (rkbar-gamma1)*z);

miu = (betta*DELTA/(1+betta*DELTA*miubar))*miu(+1) -(betta*DELTA*miubar/(1+betta*DELTA*miubar))*(sigma-1)*(c(+1) - c);

%2 Consumption of Rule of Thumbers
%c_r = mup*(w + h - (1/(1-taubar))*tau_w) - ybar/cbar*tau;

%3 Aggregate Consumption
%c = (1-lambda)*c_o; % + lambda*c_r;

%4 Wage Phillips Curve
pi_w = betta*DELTA*pi_w(+1) + kappa_w*(eta*h + sigma*c + (tau_w/(1-taubar)) - w);

%5 Wages(Real)
w = w(-1) + pi_w - pi;

%6 Price Phillips Curve
pi = betta*DELTA*pi(+1) + kappa_p*mc;

%7 Marginal Cost
mc = (1-zeta)*w + zeta*rk;

%8 Rental Rate of Capital
rk = mc + (zeta - 1)*(z + k - h);
%rk = gamma2/gamma1*z;

%9 Capital Utilization
z = (1/(gamma2/gamma1 + 1/(1-zeta)))*((1/(1-zeta))*y - (1/(1-zeta))*k + w);

%11 Production Function
y = zeta*k  + zeta*z + (1-zeta)*h;

%12 Capital Accummulation implied by optimizing household's investment
k_o = (1-delta)*k_o(-1) + ibar/kbar*i_o(-1);

%13 Aggregate capital
k = (1-lambda)*k_o;

%14 Investment of Optimizing households
i_o = (1/(1+DELTA*betta))*i_o(-1) + (1/(kappa*(1+DELTA*betta)))*q + (DELTA*betta/(1+DELTA*betta))*i_o(+1);

%15 Aggregate Investment
i = (1-lambda)*i_o;

%16 Tobin's Q
q = DELTA*betta*(1-delta)*q(+1) - (r - pi(+1))  + DELTA*betta*gamma1*rk(+1);

%17 Taylor Rule
r = phipi*pi + phiy*y;

%18 Gov't Budget Constraint
b = Rbar*b(-1) + Rbar*(g - tau - wbar*taubar*(hbar/ybar)*w - wbar*taubar*(hbar/ybar)*h - wbar*(hbar/ybar)*tau_w) + bbar*(r - Rbar*pi);

%19 Lump-sum Taxes
tau = phib*b(-1) - tau_r;

%20Market Clearing
y = (cbar/ybar)*c + (ibar/ybar)*i + g + (gamma1*kbar/ybar)*z;

%21 Gov't spending
g = rhog*g(-1) + 0*eps_taur;

%22 Transfers
tau_r = rhog*tau_r(-1) + eps_taur;

%23 Labor income taxes
tau_w = rhotau*tau_w(-1) - (ybar/(wbar*hbar))*0*eps_taur;

%24 Real Interest Rate
rr = r - pi(+1);
end;


resid;
steady;

check;

shocks;
var eps_taur;
stderr 1;
end; 



stoch_simul(periods=1000, irf = 30, noprint, nograph) y c i b z k pi r rr g tau_w tau_r mc w rk h;



output(:,ii)         = oo_.irfs.y_eps_taur;
consumption(:,ii)    = oo_.irfs.c_eps_taur;
investment(:,ii)     = oo_.irfs.i_eps_taur;
debtToGDP(:,ii)      = oo_.irfs.b_eps_taur;
capital_Util(:,ii)   = oo_.irfs.z_eps_taur;
capital(:,ii)        = oo_.irfs.k_eps_taur;
inflation(:,ii)      = 4*oo_.irfs.pi_eps_taur;
nominal_rate(:,ii)   = 4*oo_.irfs.r_eps_taur;
real_rate(:,ii)      = 4*oo_.irfs.rr_eps_taur;
spending(:,ii)       = oo_.irfs.g_eps_taur;
laborIncomeTax(:,ii) = oo_.irfs.tau_w_eps_taur;
lumpSumTransfer(:,ii)= oo_.irfs.tau_r_eps_taur;
marginal_cost(:,ii)  = oo_.irfs.mc_eps_taur;
realwage(:,ii)  = oo_.irfs.w_eps_taur;
rentalrate(:,ii)  = oo_.irfs.rk_eps_taur; 
labor(:,ii)      = oo_.irfs.h_eps_taur; 
ii = ii + 1;
end;

% plot IRFs
figure(1)
subplot(3,3,1)
plot(output)
title('Output','interpreter','latex')
legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')

subplot(3,3,2)
plot(consumption)
title('Consumption','interpreter','latex')

subplot(3,3,3)
plot(investment)
title('Investement','interpreter','latex')

subplot(3,3,4)
plot(debtToGDP)
title('Debt','interpreter','latex')

subplot(3,3,5)
plot(capital_Util)
title('Capital Utilization','interpreter','latex')

subplot(3,3,6)
plot(capital)
title('Capital','interpreter','latex')

subplot(3,3,7)
plot(inflation)
title('Inflation','interpreter','latex')
xlabel('quarters','interpreter','latex')

subplot(3,3,8)
plot(nominal_rate)
title('Nominal Interst Rate','interpreter','latex')
xlabel('quarters','interpreter','latex')

subplot(3,3,9)
plot(real_rate)
title('Real Interst Rate','interpreter','latex')
xlabel('quarters','interpreter','latex')


if tt == 1
sgtitle('BY Model - 1std Positive Government Spending Shock','interpreter','latex')


%% Compute impact and cumulative multipliers

%Multg = zeros(40,2);

for j = 1:30;
Multg094 = cumsum(prod(1./(1+nominal_rate(1:j,1))).*output(1:j,1))./cumsum(prod(1./(1+nominal_rate(1:j,1))).*spending(1:j,1));
Multg098 = cumsum(prod(1./(1+nominal_rate(1:j,2))).*output(1:j,2))./cumsum(prod(1./(1+nominal_rate(1:j,2))).*spending(1:j,2));
end;


display([' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' 'IM' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '8qrt CM' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '12qrt CM'])
disp(['Delta=0.94:' ' ' ' ' num2str(Multg094(1,1)) ' ' ' ' ' ' ' ' num2str(Multg094(8,1)) ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' num2str(Multg094(12,1))])
disp(['Delta=0.98:' ' ' ' ' num2str(Multg098(1,1)) ' ' ' ' ' ' ' ' num2str(Multg098(8,1)) ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' num2str(Multg098(12,1))])
display([' ' ' ' ' ' ' ' ' ' ' '])
display([' ' ' ' ' ' ' ' ' ' ' '])

figure(2)
subplot(4,1,1)
plot(marginal_cost)
title('Marginal Cost','interpreter','latex')
legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,2)
plot(realwage)
title('Real Wage','interpreter','latex')
%legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,3)
plot(rentalrate)
title('Rental Rate of Capital','interpreter','latex')
grid on
subplot(4,1,4)
plot(labor)
title('Labor','interpreter','latex')
grid on

elseif tt == 2
sgtitle('BY Model - 1std Positive Labour Income Tax Shock','interpreter','latex')

figure(2)
subplot(4,1,1)
plot(marginal_cost)
title('Marginal Cost','interpreter','latex')
legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,2)
plot(realwage)
title('Real Wage','interpreter','latex')
%legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,3)
plot(rentalrate)
title('Rental Rate of Capital','interpreter','latex')
grid on
subplot(4,1,4)
plot(labor)
title('Labor','interpreter','latex')
grid on

elseif tt == 3
sgtitle('BY Model - 1std Positive Transfer Shock','interpreter','latex')
figure(2)
subplot(4,1,1)
plot(marginal_cost)
title('Marginal Cost','interpreter','latex')
legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,2)
plot(realwage)
title('Real Wage','interpreter','latex')
%legend('Survival probability = 0.94', 'Survival probability = 0.98','interpreter','latex')
grid on
subplot(4,1,3)
plot(rentalrate)
title('Rental Rate of Capital','interpreter','latex')
grid on
subplot(4,1,4)
plot(labor)
title('Labor','interpreter','latex')
grid on
end;

// Save the data

output094          = output(:,1);
consumption094     = consumption(:,1);
investment094      = investment(:,1);
debtToGDP094       = debtToGDP(:,1);
capital_Util094    = capital_Util(:,1);
capital094         = capital(:,1);
inflation094       = inflation(:,1);
nominal_rate094    = nominal_rate(:,1);
real_rate094       = real_rate(:,1);
spending094        = spending(:,1);
laborIncomeTax094  = laborIncomeTax(:,1);
lumpSumTransfer094 = lumpSumTransfer(:,1);

output098          = output(:,2);
consumption098     = consumption(:,2);
investment098      = investment(:,2);
debtToGDP098       = debtToGDP(:,2);
capital_Util098    = capital_Util(:,2);
capital098         = capital(:,2);
inflation098       = inflation(:,2);
nominal_rate098    = nominal_rate(:,2);
real_rate098       = real_rate(:,2);
spending098        = spending(:,2);
laborIncomeTax098  = laborIncomeTax(:,2);
lumpSumTransfer098 = lumpSumTransfer(:,2);

if tt == 1;
DATA_BY = [output094,consumption094,investment094,debtToGDP094,capital_Util094,capital094,inflation094,nominal_rate094,real_rate094,spending094,laborIncomeTax094,lumpSumTransfer094,output098,consumption098,investment098,debtToGDP098,capital_Util098,capital098,inflation098,nominal_rate098,real_rate098,spending098,laborIncomeTax098,lumpSumTransfer098];

DATA_BY_Spending= array2table(DATA_BY,'VariableNames',{'Output096','consumption094','investment094','debtToGDP094','capital_Util094','capital094','inflation094','nominal_rate094','real_rate094','spending094','laborIncomeTax094','lumpSumTransfer094','output098','consumption098','investment098','debtToGDP098','capital_Util098','capital098','inflation098','nominal_rate098','real_rate098','spending098','laborIncomeTax098','lumpSumTransfer098'});
writetable(DATA_BY_Spending,'BY_Spending.xls');

elseif tt == 2;
DATA_BY = [output094,consumption094,investment094,debtToGDP094,capital_Util094,capital094,inflation094,nominal_rate094,real_rate094,spending094,laborIncomeTax094,lumpSumTransfer094,output098,consumption098,investment098,debtToGDP098,capital_Util098,capital098,inflation098,nominal_rate098,real_rate098,spending098,laborIncomeTax098,lumpSumTransfer098];

DATA_BY_IncomeTax= array2table(DATA_BY,'VariableNames',{'Output096','consumption094','investment094','debtToGDP094','capital_Util094','capital094','inflation094','nominal_rate094','real_rate094','spending094','laborIncomeTax094','lumpSumTransfer094','output098','consumption098','investment098','debtToGDP098','capital_Util098','capital098','inflation098','nominal_rate098','real_rate098','spending098','laborIncomeTax098','lumpSumTransfer098'});
writetable(DATA_BY_IncomeTax,'BY_IncomeTax.xls');

elseif tt == 3;
DATA_BY = [output094,consumption094,investment094,debtToGDP094,capital_Util094,capital094,inflation094,nominal_rate094,real_rate094,spending094,laborIncomeTax094,lumpSumTransfer094,output098,consumption098,investment098,debtToGDP098,capital_Util098,capital098,inflation098,nominal_rate098,real_rate098,spending098,laborIncomeTax098,lumpSumTransfer098];

DATA_BY_Transfers= array2table(DATA_BY,'VariableNames',{'Output096','consumption094','investment094','debtToGDP094','capital_Util094','capital094','inflation094','nominal_rate094','real_rate094','spending094','laborIncomeTax094','lumpSumTransfer094','output098','consumption098','investment098','debtToGDP098','capital_Util098','capital098','inflation098','nominal_rate098','real_rate098','spending098','laborIncomeTax098','lumpSumTransfer098'});
writetable(DATA_BY_Transfers,'BY_Transfers.xls');

end;

%% csvwrite('BY_Spending',DATA_BY)